from datetime import datetime
from pathlib import Path

import skyfield.api as skyfield_api

from .base import spo_types
from .base.models import TLE_FILES_DIRECTORY, TLEFile, TLEObject, de421
from .base.outputs import ObservationObject, PositionObject
from .settings import CELESTRAK_ACTIVE_TLE_URL, CELESTRAK_WEATHER_TLE_URL


class TLEFileWeather(TLEFile):
    directory: Path = TLE_FILES_DIRECTORY
    default_url = CELESTRAK_WEATHER_TLE_URL
    name = "weather_satellites"


class TLEFileSatelliteActive(TLEFile):
    directory: Path = TLE_FILES_DIRECTORY
    default_url = CELESTRAK_ACTIVE_TLE_URL
    name = "actives_satellites"


class SatelliteWeather(TLEObject):
    """
    This object allow to work with weather satellites.
    """

    tle_file_object: type[TLEFile] = TLEFileWeather


class SatelliteActive(TLEObject):
    """
    This object allow to work with actives satellites.
    """

    tle_file_object: type[TLEFile] = TLEFileSatelliteActive


class PlanetObserver:
    """
    This object allow planets observations.
    """

    directory: Path = TLE_FILES_DIRECTORY
    planets = de421
    planets_names = [
        "JUPITER",
        "MARS",
        "MERCURY",
        "MOON",
        "NEPTUNE",
        "PLUTO",
        "SATURN",
        "SUN",
        "URANUS",
        "VENUS",
    ]

    def __init__(
        self, observation_position_latitude, observation_position_longitude
    ):
        (
            self.observation_position_latitude,
            self.observation_position_longitude,
        ) = (
            observation_position_latitude,
            observation_position_longitude,
        )
        bluffton = skyfield_api.wgs84.latlon(
            self.observation_position_latitude,
            self.observation_position_longitude,
        )

        earth: spo_types.VectorSum = self.planets["earth"]  # type: ignore
        self.ssb_bluffton: spo_types.VectorSum = earth + bluffton

    def get_position(
        self, planetname: str, date: datetime | None = None
    ) -> PositionObject:
        """
        get the satelit position.
        """

        if planetname.upper() not in self.planets_names:
            raise Exception(f"< {planetname} > planet is not provided")
        try:
            planet: spo_types.ChebyshevPosition = self.planets[planetname]  # type: ignore
        except KeyError:
            planet: spo_types.ChebyshevPosition = self.planets[
                f"{planetname.upper()}_BARYCENTER"
            ]  # type: ignore
        if date is None:
            t: spo_types.Time = skyfield_api.load.timescale().now()
        else:
            date = date.replace(tzinfo=skyfield_api.utc)
            t: spo_types.Time = skyfield_api.load.timescale().from_datetime(
                date
            )

        position: spo_types.Astrometric = (
            self.planets["earth"].at(t).observe(planet)  # type: ignore
        )
        lat_lon: tuple[
            spo_types.Angle, spo_types.Angle
        ] = skyfield_api.wgs84.latlon_of(position)
        latitude = lat_lon[0]
        longitude = lat_lon[1]
        height: spo_types.Distance = skyfield_api.wgs84.height_of(position)
        return PositionObject(
            longitude=float(longitude.degrees),  # type: ignore
            latitude=float(latitude.degrees),  # type: ignore
            altitude_kms=float(height.km),  # type: ignore
            timeposition_utc=t.utc_iso(),  # type: ignore
        )

    def get_observation(
        self, planetname: str, date: datetime | None = None
    ) -> ObservationObject:

        if planetname.upper() not in self.planets_names:
            raise Exception(f"< {planetname} > planet is not provided")
        try:
            planet: spo_types.ChebyshevPosition = self.planets[
                planetname
            ]  # type: ignore
        except KeyError:
            planet: spo_types.ChebyshevPosition = self.planets[
                f"{planetname.upper()}_BARYCENTER"
            ]  # type: ignore

        ts: spo_types.Timescale = skyfield_api.load.timescale()

        if date:
            date = date.replace(tzinfo=skyfield_api.utc)
        t: spo_types.Time = ts.from_datetime(date) if date else ts.now()
        subpoint: spo_types.Astrometric = self.ssb_bluffton.at(t).observe(  # type: ignore
            planet
        )

        app: spo_types.Apparent = subpoint.apparent()
        _: tuple[
            spo_types.Angle, spo_types.Angle, spo_types.Distance
        ] = app.altaz()
        return ObservationObject(
            degrees_vertical=float(_[0].degrees),  # type: ignore
            degrees_horizontal=float(_[1].degrees),  # type: ignore
            distance_kms=float(_[2].km),  # type: ignore
            timeposition_utc=t.utc_iso(),  # type: ignore
        )



from typing import Literal

from skyfield.api import EarthSatellite
from skyfield.jpllib import ChebyshevPosition
from skyfield.positionlib import Apparent, Astrometric
from skyfield.timelib import Time, Timescale
from skyfield.toposlib import GeographicPosition
from skyfield.units import Angle, Distance
from skyfield.vectorlib import VectorSum

ORBIT_DIRECTION = Literal["GEOSTATIONARY", "E_TO_W", "W_TO_E"]



from datetime import datetime, timedelta
from operator import gt, lt
from pathlib import Path

import numpy as np
import skyfield.api as skyfield_api

from ..settings import TLE_FILES_DIRECTORY
from . import spo_types
from .decorators import deprecated
from .outputs import (
    ForecastListObject,
    ForecastObject,
    InfosObject,
    ObservationObject,
    OrbitObject,
    PositionObject,
)

load_de421 = skyfield_api.Loader(str(TLE_FILES_DIRECTORY))
de421 = load_de421("de421.bsp")  # the planets file


class TLEFile:
    directory: Path
    default_url: str
    name: str

    def __init__(self) -> None:
        today = datetime.now()
        self.filename = (
            f"{self.name}_{today.year}_{today.month}_{today.day}.txt"
        )
        self.filepath = self.directory / self.filename
        self._satellites: list[
            spo_types.EarthSatellite
        ] = skyfield_api.load.tle_file(
            self.default_url,
            filename=str(self.filepath),
            reload=self.filepath.exists() is False,
        )
        self.satellites: dict[str, spo_types.EarthSatellite] = {
            sat.name: sat for sat in self._satellites
        }
        self.tle_content = [
            i.strip() for i in self.filepath.read_text().splitlines()
        ]

    def list(self):
        """
        List the supported objects by name.
        """
        return sorted(list(self.satellites.keys()))


class TLEObject:
    tle_file_object: type[TLEFile]

    def __init__(
        self,
        satellite_name: str,
        observation_position_longitude: float = 0.0,
        observation_position_latitude: float = 0.0,
    ):
        self.tle_file: TLEFile = self.tle_file_object()
        if satellite_name not in self.tle_file.satellites:
            raise Exception(f"< {satellite_name} > not in tle file.")
        self.satellite_name = satellite_name
        self.observation_position_longitude = observation_position_longitude
        self.observation_position_latitude = observation_position_latitude

        self.satellite: spo_types.EarthSatellite = self.tle_file.satellites[
            self.satellite_name
        ]
        index = self.tle_file.tle_content.index(self.satellite_name)
        self.two_lines_elements = self.tle_file.tle_content[
            index + 1 : index + 3
        ]
        rotations_number = None
        self.direction: spo_types.ORBIT_DIRECTION = "GEOSTATIONARY"

        line = self.two_lines_elements[-1]
        _a, _b = line.split()[-1], line.split()[-2]
        rotations_number = float(_a) if "." in _a else float(_b)

        self.satellite_rotation_seconds: float = 0.0
        if rotations_number:
            self.direction = (
                "W_TO_E" if float(line.split()[2]) < 90 else "E_TO_W"
            )
            self.satellite_rotation_seconds = 24 / rotations_number * 3600
        bluffton: spo_types.GeographicPosition = skyfield_api.wgs84.latlon(
            self.observation_position_latitude,
            self.observation_position_longitude,
        )

        earth: spo_types.VectorSum = de421["earth"]  # type: ignore
        self.ssb_bluffton: spo_types.VectorSum = earth + bluffton
        self.ssb_satellite: spo_types.VectorSum = earth + self.satellite

    @property
    @deprecated
    def satellite_rotation_direction(self):
        return self.direction

    def get_infos(self):
        return InfosObject(
            name=self.satellite_name,
            direction=self.direction,
            rotation_seconds=self.satellite_rotation_seconds,
            tle=self.two_lines_elements,
        )

    def get_position(self, date: datetime | None = None) -> PositionObject:
        """
        get the satelit position.
        """
        t: spo_types.Time
        if date is None:
            t = skyfield_api.load.timescale().now()
        else:
            date = date.replace(tzinfo=skyfield_api.utc)
            t = skyfield_api.load.timescale().from_datetime(date)

        subpoint: spo_types.GeographicPosition = skyfield_api.wgs84.subpoint(
            self.satellite.at(t)
        )

        return PositionObject(
            longitude=float(subpoint.longitude.degrees),  # type: ignore
            latitude=float(subpoint.latitude.degrees),  # type: ignore
            altitude_kms=float(subpoint.elevation.km),  # type: ignore
            timeposition_utc=t.utc_iso(),  # type: ignore
        )

    def get_observation(
        self, date: datetime | None = None
    ) -> ObservationObject:
        """
        get observation data from the ground.
        return the x and y angle from your position.
        """
        t: spo_types.Time
        if date is None:
            t = skyfield_api.load.timescale().now()
        else:
            date = date.replace(tzinfo=skyfield_api.utc)
            t = skyfield_api.load.timescale().from_datetime(date)
        topocentric2: spo_types.Astrometric = self.ssb_bluffton.at(t).observe(
            self.ssb_satellite
        )
        app: spo_types.Apparent = topocentric2.apparent()
        _: tuple[
            spo_types.Angle, spo_types.Angle, spo_types.Distance
        ] = app.altaz()
        return ObservationObject(
            degrees_vertical=float(_[0].degrees),  # type: ignore
            degrees_horizontal=float(_[1].degrees),  # type: ignore
            distance_kms=float(_[2].km),  # type: ignore
            timeposition_utc=t.utc_iso(),  # type: ignore
        )

    def get_orbit(self, date: datetime | None = None) -> OrbitObject:
        """
        get the orbit of the object.

        Each point will have a step of 10 seconds.
        """
        t: datetime
        if date is None:
            t = datetime.utcnow()
        else:
            t = date

        seconds: np.ndarray = np.arange(
            0, round(self.satellite_rotation_seconds), 10
        )  # 1h32
        ts: spo_types.Timescale = skyfield_api.load.timescale(builtin=True)
        times: spo_types.Time = ts.utc(
            t.year, t.month, t.day, t.hour, t.minute, seconds
        )  # type: ignore
        subpoint: spo_types.GeographicPosition = skyfield_api.wgs84.subpoint(
            self.satellite.at(times)
        )
        orbit = list(
            zip(
                subpoint.longitude.degrees,  # type: ignore
                subpoint.latitude.degrees,  # type: ignore
                subpoint.elevation.km,  # type: ignore
                times.utc_iso(),
            )
        )
        orbit_one: list[PositionObject] = []
        orbit_two: list[PositionObject] = []
        _oper = lt if self.direction == "W_TO_E" else gt
        test = False
        for idx, elem in enumerate(orbit):
            try:
                posobj = PositionObject(
                    longitude=float(elem[0]),
                    latitude=float(elem[1]),
                    altitude_kms=float(elem[2]),
                    timeposition_utc=elem[3],
                )
                if _oper(orbit[idx + 1][0], elem[0]):
                    test = True
                if test:
                    orbit_two.append(posobj)
                else:
                    orbit_one.append(posobj)
            except IndexError:
                break
        if orbit_two:
            orbit_one.append(orbit_two.pop(0))
        return OrbitObject(
            part_one=orbit_one,
            part_two=orbit_two,
            direction=self.direction,
        )

    def get_observations_forecast_24H(
        self,
        altitude_degrees=15,
        include_observation=True,
        include_position=True,
    ) -> ForecastListObject:
        """
        Return the possible observations for the observer position
        with a forecast of 24 hours.
        """

        def format_data(ti):
            topocentric2: spo_types.Astrometric = self.ssb_bluffton.at(
                ti
            ).observe(
                self.ssb_satellite
            )  # type: ignore
            app: spo_types.Apparent = topocentric2.apparent()
            _: tuple[
                spo_types.Angle, spo_types.Angle, spo_types.Distance
            ] = app.altaz()
            angle = round(_[0].degrees)  # type: ignore
            ti_str = ti.utc_strftime("%Y-%m-%dT%H:%M:%SZ")
            res = {
                "time": ti_str,
                "is_sunlit": bool(self.satellite.at(ti).is_sunlit(de421)),
                "angle_x": angle,
            }

            if include_position:
                subpoint = skyfield_api.wgs84.subpoint(self.satellite.at(ti))
                res["position"] = PositionObject(
                    longitude=float(subpoint.longitude.degrees),  # type: ignore
                    latitude=float(subpoint.latitude.degrees),  # type: ignore
                    altitude_kms=float(subpoint.elevation.km),  # type: ignore
                    timeposition_utc=ti_str,
                )

            if include_observation:
                res["observation"] = ObservationObject(
                    degrees_vertical=float(_[0].degrees),  # type: ignore
                    degrees_horizontal=float(_[1].degrees),  # type: ignore
                    distance_kms=float(_[2].km),  # type: ignore
                    timeposition_utc=ti_str,
                )

            return ForecastObject(**res)

        bluffton: spo_types.GeographicPosition = skyfield_api.wgs84.latlon(
            self.observation_position_latitude,
            self.observation_position_longitude,
        )
        utcnow = datetime.utcnow().replace(tzinfo=skyfield_api.utc)
        futur = utcnow + timedelta(days=1)
        futur = futur.replace(hour=23, minute=59, second=59)
        t0: spo_types.Time = skyfield_api.load.timescale().from_datetime(
            utcnow
        )
        t1: spo_types.Time = skyfield_api.load.timescale().from_datetime(futur)
        times: spo_types.Time
        times, _ = self.satellite.find_events(
            bluffton, t0, t1, altitude_degrees=altitude_degrees
        )
        return ForecastListObject(
            sorted(
                [format_data(ti) for ti in times],
                key=lambda x: x.time,
            )
        )  # type: ignore



import os
from pathlib import Path

TLE_FILES_DIRECTORY = Path(os.getenv("TLE_FILES_DIRECTORY", "tle_files"))
TLE_FILES_DIRECTORY.mkdir(exist_ok=True, parents=True)

CELESTRAK_ACTIVE_TLE_URL = (
    "https://www.celestrak.com/NORAD/elements/active.txt"
)
CELESTRAK_WEATHER_TLE_URL = (
    "https://www.celestrak.com/NORAD/elements/weather.txt"
)



from typing import Literal

import numpy as np
from matplotlib import pyplot as plt


def get_x(angle_degree: float):
    return np.pi * angle_degree / 180


def draw_compass(
    angle_degree,
    title,
    direction="N",
    orientation: Literal["H", "V"] = "H",
):
    """
    Simple matplotlib compass drawwing method.
    """
    plt.rcParams["figure.figsize"] = [7.00, 3.50]
    plt.rcParams["figure.autolayout"] = True
    ax = plt.subplot(1, 1, 1, projection="polar")
    ax.set_theta_direction(-1)  # type: ignore
    ax.set_yticklabels([])
    if orientation == "V":
        angle_degree = angle_degree - 90
        ax.set_xticklabels(
            [
                "90°",
                "",
                "",
                "",
                "-90°",
                "-45°",
                "0°",
                "45°",
            ]
        )
    else:
        ax.set_xticklabels(
            [
                "N",
                "",
                "E",
                "",
                "S",
                "",
                "W",
                "",
            ]
        )
    ax.set_theta_zero_location(direction)  # type: ignore
    ax.grid(False)
    ax.arrow(
        x=get_x(angle_degree),
        y=0,
        dx=0,
        dy=1,
        width=0.05,
        facecolor="red",
        edgecolor="none",
    )
    plt.title(title)
    plt.show()



import warnings
from functools import wraps


def deprecated(fn):
    @wraps(fn)
    def wrapped(*args, **kwargs):
        warnings.warn(
            f"{fn.__name__} is deprecated and will be removed in the futur.",
            DeprecationWarning,
        )
        return fn(*args, **kwargs)

    return wrapped



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['space_objects', 'space_objects.base', 'space_objects.matplotlib']

package_data = \
{'': ['*']}

install_requires = \
['skyfield>=1.45,<2.0', 'tabulate>=0.9.0,<0.10.0']

setup_kwargs = {
    'name': 'space-objects',
    'version': '1.2.0',
    'description': 'wrapper lib around the skyfield package',
    'long_description': '# space_objects\n\n- ![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)\n\n- ![coverage](coverage-badge.svg)\n\n```bash\npip install space-objects\n```\n\n## Presentation\n\nVery simple objects which provides methods to get position and observation for space objects like the ISS station.\n\n- Allow to get position, observation and orbit data for actives and weather satellites.\n- Allow to get position, observation data for planets.\n\nThis project use the great [skyfield package](https://github.com/skyfielders/python-skyfield),  thanks to it\'s maintainers.\n\n## Motivation\n\nI was working on a project of an ISS tracker build on a rasperry pi and servo motors.\n\nI needed the ISS observation angles degrees for a given position, the position of the ISS in real time, this ISS orbit and a list of the forecast possible observations from my position.\n\nI needed to be web service agnostic and to be autonomous for the needed data.\n\nNow than this project is ready i decided to clean my researches and algos in appropriates repositories in the hope there will be usefull.\n\n## Environment Variable\n\nThis library support one environment variable, this variable is not mandatory.\n\nThe skyfield library will load some files, like TLE files and de421.bsp, by default space_objects create a directory `tle_files` in your script path and files will be download in this directory.\n\nYou can easily overide this behaviour with an environment variable: `TLE_FILES_DIRECTORY=< you custom path >`\n\n## Samples\n\n### Get the ISS position\n\n```python\nfrom space_objects.models import SatelliteActive\n\nmy_position = {\n    "observation_position_longitude": 1.433333,\n    "observation_position_latitude": 43.6,\n}\n\niss_object = SatelliteActive(satellite_name="ISS (ZARYA)", **my_position)\nposition = iss_object.get_position()\nprint(position)\n```\n\n```python\nPositionObject(\n  longitude=95.42870673510215,\n  latitude=-45.701086600061714,\n  altitude_kms=433.23953982510665,\n  timeposition_utc=\'2022-10-31T12:48:35Z\'\n)\n\n```\n\n### Get the ISS observation\n\n```python\nfrom space_objects.models import SatelliteActive\n\nmy_position = {\n    "observation_position_longitude": 1.433333,\n    "observation_position_latitude": 43.6,\n}\n\niss_object = SatelliteActive(satellite_name="ISS (ZARYA)", **my_position)\nobservation = iss_object.get_observation()\nprint(observation)\n```\n\n```python\nObservationObject(\n  degrees_vertical=-59.92007436232538,\n  degrees_horizontal=124.47513589616938,\n  distance_kms=11504.317997710621,\n  timeposition_utc=\'2022-10-31T12:48:35Z\'\n)\n```\n\n### Get the ISS informations\n\n```python\nfrom space_objects.models import SatelliteActive\n\nmy_position = {\n    "observation_position_longitude": 1.433333,\n    "observation_position_latitude": 43.6,\n}\n\niss_object = SatelliteActive(satellite_name="ISS (ZARYA)", **my_position)\ninfos = iss_object.get_infos()\nprint(infos)\n```\n\n```python\nInfosObject(\n  name=\'ISS (ZARYA)\',\n  direction=\'W_TO_E\',\n  rotation_seconds=5575.255143989082,\n  tle=[\n    \'1 25544U 98067A   22307.39895310  .00017310  00000+0  31368-3 0  9997\',\n    \'2 25544  51.6453   6.9445 0006467  35.6599  53.8579 15.49704861366775\'\n  ]\n)\n```\n\n### Get the 24 hours forecast positions and observations for the ISS\n\n```python\nfrom space_objects.models import SatelliteActive\n\nmy_position = {\n    "observation_position_longitude": 1.433333,\n    "observation_position_latitude": 43.6,\n}\n\niss_object = SatelliteActive(satellite_name="ISS (ZARYA)", **my_position)\nforecast_observations = iss_object.get_observations_forecast_24H()\nforecast_observations.tprint()\n```\n\n```text\ntimeposition_utc        degrees_vertical  is_sunlit\n--------------------  ------------------  -----------\n2022-11-01T01:41:39Z                  15  False\n2022-11-01T01:44:19Z                  85  False\n2022-11-01T01:47:00Z                  15  False\n2022-11-01T03:19:29Z                  15  False\n2022-11-01T03:21:27Z                  24  False\n2022-11-01T03:23:25Z                  15  False\n2022-11-01T04:57:16Z                  15  False\n2022-11-01T04:59:00Z                  21  True\n2022-11-01T05:00:43Z                  15  True\n2022-11-01T06:33:41Z                  15  True\n2022-11-01T06:36:18Z                  52  True\n2022-11-01T06:38:54Z                  15  True\n2022-11-01T08:11:01Z                  15  True\n2022-11-01T08:12:53Z                  23  True\n2022-11-01T08:14:45Z                  15  True\n```\n\n```python\nforecast_observation = forecast_observations[10].observation\nforecast_position = forecast_observations[10].position\nprint(forecast_position)\nprint(forecast_observation)\n```\n\n```text\nPositionObject(\n  longitude=3.2171481263959283,\n  latitude=46.0182975420938,\n  altitude_kms=422.3427194167064,\n  timeposition_utc=\'2022-11-01T06:36:18Z\'\n)\nObservationObject(\n  degrees_vertical=52.03750100701985,\n  degrees_horizontal=27.071376289371536,\n  distance_kms=525.8886708185943,\n  timeposition_utc=\'2022-11-01T06:36:18Z\'\n)\n```\n\n![forecast_vertical_orientation](assets/images/forecast_vertical_orientation.png)\n![forecast_horyzontal_orientation](assets/images/forecast_horyzontal_orientation.png)\n\n### List available satellites\n\n```python\nfrom space_objects.models import TLEFileSatelliteActive, TLEFileWeather\n\nsatellites_actives = TLEFileSatelliteActive().list()\nsatellites_weather = TLEFileWeather().list()\nprint(satellites_actives)\nprint(satellites_weather)\n```\n\n### List available planets\n\n```python\nfrom space_objects.models import PlanetObserver\n\nplanets = PlanetObserver.planets_names\nprint(planets)\n```\n\n### Get the Sun position and observation\n\n```python\nfrom space_objects.models import PlanetObserver\n\nmy_position = {\n    "observation_position_longitude": 1.433333,\n    "observation_position_latitude": 43.6,\n}\n\nplanet_observer = PlanetObserver(**my_position)\nsun_observation = planet_observer.get_observation("SUN")\nsun_position = planet_observer.get_position("SUN")\n\nprint(sun_observation)\nprint(sun_position)\n```\n\n```text\nObservationObject(\n  degrees_vertical=27.123628441099548,\n  degrees_horizontal=209.69557585058428,\n  distance_kms=148520276.74009204,\n  timeposition_utc=\'2022-10-31T13:26:06Z\'\n)\nPositionObject(\n  longitude=-25.61420206366428,\n  latitude=-14.209948450817524,\n  altitude_kms=148516819.7673485,\n  timeposition_utc=\'2022-10-31T13:26:06Z\'\n)\n```\n\n![vertical_orientation_sun](assets/images/vertical_orientation_sun.png)\n![horyzontal_orientation_sun](assets/images/horyzontal_orientation_sun.png)\n\n## Notebooks\n\n- we maintain some Jupyter notebooks for concretes samples and demonstrations.\n\n## Development\n\n- this project use `poetry`\n- the code quality tools used are:\n  - `black`\n  - `isort`\n  - `flake8`\n  - `coverage`\n  - `pytest`\n  - `genbadge`\n\n- a precommit hook may be installed with the `pre-commit` python package\n- the pre commit hook will play:\n  - `black`\n  - `isort`\n  - `flake8`\n  - `coverage run -m pytest -W ignore`\n  - `coverage xml`\n  - `genbadge coverage -i coverage.xml`\n\n### install the development environment\n\n```bash\npoetry install\npre-commit install\n```\n\n### Getting Started\n\n```bash\npoetry shell\nexport PYTHONPATH=.\n```\n\n## Authors\n\n- **Olivier Larrieu** - [larrieu-olivier](https://github.com/larrieu-olivier)\n',
    'author': 'larrieu',
    'author_email': 'larrieuolivierad@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass

from tabulate import tabulate

from .spo_types import ORBIT_DIRECTION


@dataclass
class InfosObject:
    """
    An object information representation.
    """

    name: str
    direction: ORBIT_DIRECTION
    rotation_seconds: float
    tle: list[str]


@dataclass
class PositionObject:
    """
    An object position representation.
    """

    longitude: float
    latitude: float
    altitude_kms: float
    timeposition_utc: str

    @property
    def simple_list(self):
        return [
            self.longitude,
            self.latitude,
            self.altitude_kms,
            self.timeposition_utc,
        ]

    def to_dict(self):
        return {
            "latitude": self.latitude,
            "longitude": self.longitude,
            "altitude_kms": self.altitude_kms,
            "timeposition_utc": self.timeposition_utc,
        }

    def tprint(self) -> None:
        """
        Tabular print for pretty display
        """
        print(
            tabulate(
                [self.simple_list],
                headers=[
                    "longitude",
                    "latitude",
                    "altitude_kms",
                    "timeposition_utc",
                ],
            )
        )


@dataclass
class ObservationObject:
    """
    An object observation representation.
    """

    degrees_vertical: float
    degrees_horizontal: float

    distance_kms: float  # the distance between the observer and the object
    timeposition_utc: str

    @property
    def simple_list(self):
        return [
            self.degrees_vertical,
            self.degrees_horizontal,
            self.distance_kms,
            self.timeposition_utc,
        ]

    def to_dict(self):
        return {
            "degrees_vertical": self.degrees_vertical,
            "degrees_horizontal": self.degrees_horizontal,
            "distance_kms": self.distance_kms,
            "timeposition_utc": self.timeposition_utc,
        }

    def tprint(self) -> None:
        """
        Tabular print for pretty display
        """
        print(
            tabulate(
                [self.simple_list],
                headers=[
                    "degrees_vertical",
                    "degrees_horizontal",
                    "distance_kms",
                    "timeposition_utc",
                ],
            )
        )


@dataclass
class ForecastObject:
    time: str
    is_sunlit: bool
    angle_x: float | int
    observation: ObservationObject
    position: PositionObject

    def to_dict(self):
        _ = {
            "degrees_vertical": self.angle_x,
            "is_sunlit": self.is_sunlit,
            "timeposition_utc": self.time,
        }
        if self.observation:
            _["observation"] = self.observation.to_dict()
        if self.position:
            _["position"] = self.position.to_dict()
        return _


class ForecastListObject(list[ForecastObject]):
    def to_dict(self):
        return [i.to_dict() for i in self]

    def tprint(self) -> None:
        """
        Tabular print for pretty display
        """
        print(
            tabulate(
                [(i.time, i.angle_x, i.is_sunlit) for i in self],
                headers=[
                    "timeposition_utc",
                    "degrees_vertical",
                    "is_sunlit",
                ],
            )
        )


@dataclass
class OrbitObject:
    """
    An orbit object representation.

    An orbit is divided in two part, before and after meridian.
    """

    part_one: list[PositionObject]
    part_two: list[PositionObject]
    direction: ORBIT_DIRECTION

    @property
    def date_end(self):
        _ = max(
            [
                self.part_one[0],
                self.part_one[-1],
                self.part_two[0],
                self.part_two[-1],
            ],
            key=lambda e: e.timeposition_utc,
        )
        return _.timeposition_utc

    @property
    def date_start(self):
        _ = min(
            [
                self.part_one[0],
                self.part_one[-1],
                self.part_two[0],
                self.part_two[-1],
            ],
            key=lambda e: e.timeposition_utc,
        )
        return _.timeposition_utc

    def to_dict(self):
        return {
            "part_one": [i.to_dict() for i in self.part_one],
            "part_two": [i.to_dict() for i in self.part_two],
            "date_start": self.date_start,
            "date_end": self.date_end,
            "direction": self.direction,
        }
